/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.4
  * @date     2021-11-26
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_board.h"
#include "at32f403a_407_clock.h"

#define FLASH_ADDR                       ((uint32_t)(0x08000000 + (128 * 1024)))
#define BUFFER_SIZE                      ((uint32_t)(32 * 1024))
#define SECTOR_SIZE                      ((uint32_t)(2 * 1024))

uint8_t dst_buffer1[BUFFER_SIZE];
uint8_t dst_buffer2[BUFFER_SIZE];

confirm_state buffer1_completed = FALSE, buffer2_completed = FALSE;

dma_init_type dma_init_struct;

/**
  * @brief  dma configuration.
  * @param  none
  * @retval none
  */
void dma_config(void)
{
  /* enable dma1 periph clock */
  crm_periph_clock_enable(CRM_DMA1_PERIPH_CLOCK, TRUE);

  dma_default_para_init(&dma_init_struct);

  /* config dma1 channel1 */
  dma_reset(DMA1_CHANNEL1);
  dma_init_struct.buffer_size = BUFFER_SIZE / (sizeof(uint32_t));
  dma_init_struct.direction = DMA_DIR_MEMORY_TO_MEMORY;
  dma_init_struct.memory_base_addr = (uint32_t)dst_buffer1;
  dma_init_struct.memory_data_width = DMA_MEMORY_DATA_WIDTH_WORD;
  dma_init_struct.memory_inc_enable = TRUE;
  dma_init_struct.peripheral_base_addr = FLASH_ADDR;
  dma_init_struct.peripheral_data_width = DMA_PERIPHERAL_DATA_WIDTH_WORD;
  dma_init_struct.peripheral_inc_enable = TRUE;
  dma_init_struct.priority = DMA_PRIORITY_HIGH;
  dma_init(DMA1_CHANNEL1, &dma_init_struct);
  dma_interrupt_enable(DMA1_CHANNEL1, DMA_FDT_INT, TRUE);

  /* config dma1 channel2 */
  dma_reset(DMA1_CHANNEL2);
  dma_init_struct.buffer_size = BUFFER_SIZE / (sizeof(uint32_t));
  dma_init_struct.direction = DMA_DIR_MEMORY_TO_MEMORY;
  dma_init_struct.memory_base_addr = (uint32_t)dst_buffer2;
  dma_init_struct.memory_data_width = DMA_MEMORY_DATA_WIDTH_WORD;
  dma_init_struct.memory_inc_enable = TRUE;
  dma_init_struct.peripheral_base_addr = FLASH_ADDR;
  dma_init_struct.peripheral_data_width = DMA_PERIPHERAL_DATA_WIDTH_WORD;
  dma_init_struct.peripheral_inc_enable = TRUE;
  dma_init_struct.priority = DMA_PRIORITY_HIGH;
  dma_init(DMA1_CHANNEL2, &dma_init_struct);
  dma_interrupt_enable(DMA1_CHANNEL2, DMA_FDT_INT, TRUE);

  /* config nvic */
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  nvic_irq_enable(DMA1_Channel1_IRQn, 1, 0);
  nvic_irq_enable(DMA1_Channel2_IRQn, 1, 0);
}

/**
  * @brief  copy data from flash to buffer1 with dma.
  * @param  flash_addr: source address of flash
  * @param  length: data length, aligned 4 bytes
  * @retval none
  */
void dma_buffer1_start(uint32_t flash_addr, uint32_t length)
{
  dma_init_struct.buffer_size = length / (sizeof(uint32_t));
  dma_init_struct.memory_base_addr = (uint32_t)dst_buffer1;
  dma_init_struct.peripheral_base_addr = flash_addr;
  dma_init(DMA1_CHANNEL1, &dma_init_struct);

  /* enable dma1 channel1 */
  dma_channel_enable(DMA1_CHANNEL1, TRUE);
}

/**
  * @brief  copy data from flash to buffer2 with dma.
  * @param  flash_addr: source address of flash
  * @param  length: data length, aligned 4 bytes
  * @retval none
  */
void dma_buffer2_start(uint32_t flash_addr, uint32_t length)
{
  dma_init_struct.buffer_size = length / (sizeof(uint32_t));
  dma_init_struct.memory_base_addr = (uint32_t)dst_buffer2;
  dma_init_struct.peripheral_base_addr = flash_addr;
  dma_init(DMA1_CHANNEL2, &dma_init_struct);

  /* enable dma1 channel2 */
  dma_channel_enable(DMA1_CHANNEL2, TRUE);
}

/**
  * @brief  dma1 channel1 irq handler.
  * @param  none
  * @retval none
  */
void DMA1_Channel1_IRQHandler(void)
{
  if(dma_flag_get(DMA1_FDT1_FLAG) == SET)
  {
    dma_flag_clear(DMA1_GL1_FLAG | DMA1_FDT1_FLAG | DMA1_HDT1_FLAG | DMA1_DTERR1_FLAG);
    dma_channel_enable(DMA1_CHANNEL1, FALSE);
    buffer1_completed = TRUE;
  }
}

/**
  * @brief  dma1 channel2 irq handler.
  * @param  none
  * @retval none
  */
void DMA1_Channel2_IRQHandler(void)
{
  if(dma_flag_get(DMA1_FDT2_FLAG) == SET)
  {
    dma_flag_clear(DMA1_GL2_FLAG | DMA1_FDT2_FLAG | DMA1_HDT2_FLAG | DMA1_DTERR2_FLAG);
    dma_channel_enable(DMA1_CHANNEL2, FALSE);
    buffer2_completed = TRUE;
  }
}

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  uint32_t sector_index = 0, addr_index = 0;
  uint32_t sector_addr = 0, addr = 0;

  system_clock_config();

  at32_board_init();

  flash_unlock();
  for(sector_index = 0; sector_index < (BUFFER_SIZE / SECTOR_SIZE); sector_index++)
  {
    sector_addr = FLASH_ADDR + (sector_index * SECTOR_SIZE);
    flash_sector_erase(sector_addr);
    for(addr_index = 0; addr_index < (SECTOR_SIZE / (sizeof(uint32_t))); addr_index ++)
    {
      addr = sector_addr + (addr_index * sizeof(uint32_t));
      flash_word_program(addr, addr_index);
    }
  }
  flash_lock();

  dma_config();

  dma_buffer1_start(FLASH_ADDR, SECTOR_SIZE);

  while(1)
  {
    while(buffer1_completed == FALSE);
    dma_buffer2_start(FLASH_ADDR + 0x10, SECTOR_SIZE);
    // process buffer1 data
    at32_led_toggle(LED2);
    buffer1_completed = FALSE;

    delay_ms(200);

    while(buffer2_completed == FALSE);
    dma_buffer1_start(FLASH_ADDR + 0x20, SECTOR_SIZE);
    // process buffer2 data
    at32_led_toggle(LED2);
    buffer2_completed = FALSE;

    delay_ms(200);
  }
}
